Package: timeDate
Title: Rmetrics - Chronological and Calendar Objects
Version: 4052.112
Authors@R: c(person("Diethelm", "Wuertz", role="aut", comment = "original code")
	   , person("Tobias", "Setz", role = c("aut"), email = "tobias.setz@live.com")
	   , person("Yohan", "Chalabi", role = "aut")
	   , person("Martin","Maechler", role = "ctb", email = "maechler@stat.math.ethz.ch",
	     comment = c(ORCID = "0000-0002-8685-9910"))
	   , person(given = c("Joe", "W."), family = "Byers", role = "ctb")
	   , person(given = c("Georgi", "N."), family = "Boshnakov", 
                    role = c("cre", "aut"), email = "georgi.boshnakov@manchester.ac.uk",
		    comment = c(ORCID = "0000-0003-2839-346X")) )
Description: The 'timeDate' class fulfils the conventions of the ISO 8601 
	standard as well as of the ANSI C and POSIX standards. Beyond
	these standards it provides the "Financial Center" concept
	which allows to handle data records collected in different time 
	zones and mix them up to have always the proper time stamps with 
	respect to your personal financial center, or alternatively to the GMT
	reference time. It can thus also handle time stamps from historical 
	data records from the same time zone, even if the financial 
	centers changed day light saving times at different calendar
	dates.
Depends: R (>= 3.6.0), methods
Imports: graphics, utils, stats
Suggests: RUnit
License: GPL (>= 2)
Encoding: UTF-8
URL: https://geobosh.github.io/timeDateDoc/ (doc),
        https://CRAN.R-project.org/package=timeDate,
        https://www.rmetrics.org
BugReports: https://r-forge.r-project.org/tracker/?atid=633&group_id=156&func=browse
NeedsCompilation: no
Packaged: 2026-01-26 09:43:57 UTC; georgi
Author: Diethelm Wuertz [aut] (original code),
  Tobias Setz [aut],
  Yohan Chalabi [aut],
  Martin Maechler [ctb] (ORCID: <https://orcid.org/0000-0002-8685-9910>),
  Joe W. Byers [ctb],
  Georgi N. Boshnakov [cre, aut] (ORCID:
    <https://orcid.org/0000-0003-2839-346X>)
Maintainer: Georgi N. Boshnakov <georgi.boshnakov@manchester.ac.uk>
Repository: CRAN
Date/Publication: 2026-01-28 14:30:02 UTC
Built: R 4.6.0; ; 2026-04-26 08:52:14 UTC; unix
