This archive contains the distribution TimeDate,
version 2.34_03:

  Date and time formatting subroutines

This software is copyright (c) 2020 by Graham Barr.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.


This README file was generated by Dist::Zilla::Plugin::Readme v6.033.
