\name{currentYear}
\alias{currentYear}


\title{Current year}

\description{
  A variable containing the current year.
}

\note{
  It is not allowed to change this variable.
}

\examples{
## currentYear
getRmetricsOptions("currentYear")
}

\keyword{chron}
