\name{myUnits}
\alias{myUnits}


\title{Frequency of date/time units}

\description{

  A variable with the frequency of date/units.

}

\value{
  the date/time units, a character value, yy default \code{"days"}
}

\examples{
## myUnits
getRmetricsOptions("myUnits")
}

\keyword{chron}
